;******************************************************************
;                                                                 *
;   Filename: Sequencer 8-ch 16F1827.asm                          *
;     Author: Mike McLaren, K8LH                                  *
;    (C)2010: Micro Applications Consultants                      *
;       Date: 28-May-14                                           *
;                                                                 *
;                                                                 *
;   16F1827 LED Sequencer (16-MHz INTOSC) with eight active high  *
;   LEDs on PORTB and an active low push button switch (with 10k  *
;   pull up resistor) on RA0.                                     *
;                                                                 *
;                                                                 *
;      MPLab: 8.84    (tabs=8)                                    *
;      MPAsm: 5.44    (absolute mode)                             *
;                                                                 *
;******************************************************************
        #include <p16f1827.inc>
        errorlevel -302
        list st=off
        radix dec

  __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _IESO_OFF & _FCMEN_OFF
  __CONFIG _CONFIG2, _PLLEN_OFF &_BORV_HI & _LVP_OFF
;
; _PWRTE_OFF    default
; _MCLRE_ON     default
; _CP_OFF       default
; _CPD_OFF      default
; _BOREN_ON     default
; _CLKOUTEN_OFF default
;
; _WRT_OFF      default
; _STVREN_ON    default
;

        cblock  0x20
steps                   ; sequence step counter
swold                   ; switch state latch
fxnbr                   ; effect number
delayhi                 ; DelayCy() variable
duration                ; 1..255 (10..2550 msecs)
        endc

#define sw1     0       ; bit index for sw1 on RA0 pin
                        ; requires external 10K pull-up

;==================================================================
;  K8LH DelayCy() subsystem macro generates four instructions     =
;==================================================================
        radix dec
clock   equ     16              ; 4, 8, 12, 16, 20 (MHz), etc.
usecs   equ     clock/4         ; cycles/microsecond multiplier
msecs   equ     clock/4*1000    ; cycles/millisecond multiplier

DelayCy macro   delay           ; 11..327690 cycle range
        movlw   high((delay-11)/5)+1
        movwf   delayhi
        movlw   low ((delay-11)/5)
        call    uDelay-((delay-11)%5)
        endm

;******************************************************************
;  reset vector                                                   *
;******************************************************************
        org     0x000

;******************************************************************
;  main init                                                      *
;******************************************************************

init
        banksel OSCCON          ; bank 1                          |B1
        movlw   b'01111000'     ;                                 |B1
        movwf   OSCCON          ; setup 16-MHz INTOSC             |B1
waitfs  btfss   OSCSTAT,HFIOFS  ; stable? yes, skip, else         |B1
        bra     waitfs          ; branch and wait                 |B1

        banksel PORTA           ; bank 0                          |B0
        clrf    PORTA           ;                                 |B0
        clrf    PORTB           ;                                 |B0
        banksel ANSELA          ; bank 3                          |B3
        clrf    ANSELA          ; set digital I/O                 |B3
        clrf    ANSELB          ; set digital I/O                 |B3
        banksel TRISA           ; bank 1                          |B1
        movlw   1<<sw1          ; mask bit for sw1 (RA0)          |B1
        movwf   TRISA           ; RA0 input, all others outputs   |B1
        clrf    TRISB           ; PORTB all outputs               |B1
        movlb   0               ; bank 0                          |B0
;
;  init program variables
;
        movlw   -1              ;                                 |B0
        movwf   fxnbr           ; init "effect number" var        |B0
        clrf    swold           ; prep switch state latch         |B0

;******************************************************************
;  main loop                                                      *
;******************************************************************

nextfx
        clrf    PORTB           ; turn the display off            |B0
        DelayCy(50*msecs)       ; short pause between effects     |B0
        incf    fxnbr,F         ; bump FX number                  |B0
        movlw   fxtot           ; total fx directory entries      |B0
        subwf   fxnbr,W         ; is fxnbr < total?               |B0
        skpnc                   ; yes, skip, else                 |B0
        clrf    fxnbr           ; rollover 'fxnbr' to 0           |B0
;
;  get current fx sequence table address from the "fxdir" table
;
newseq
        lslf    fxnbr,W         ; FX number 0,1,2.. -> 0,2,4..    |B0
        addlw   low(fxdir)      ; add fx directory address lo     |B0
        movwf   FSR0L           ; set pointer lo                  |B0
        movlw   high(fxdir)     ; get fx directory address hi     |B0
        iorlw   0x80            ; set bit 7 for rom access        |B0
        movwf   FSR0H           ; set pointer hi                  |B0
        skpnc                   ; carry? no, skip, else           |B0
        incf    FSR0H,F         ; bump the hi pointer             |B0
;
;  copy fx table address from directory table to FSR1
;
        moviw   FSR0++          ; copy fx table address to FSR1   |B0
        movwf   FSR1L           ; fx table address lo             |B0
        moviw   FSR0++          ;                                 |B0
        iorlw   0x80            ; set bit 7 for rom access        |B0
        movwf   FSR1H           ; fx table address hi             |B0
;
;  read total number of sequence "steps" in this effects table
;
        moviw   FSR1++          ; read first byte (steps)         |B0
        movwf   steps           ; save number of steps            |B0
        skpnz                   ; 0 steps? no, skip, else         |B0
        bra     nextfx          ; branch (try next table)         |B0
;
;  start new/next sequence 'step'
;
newstep
        moviw   FSR1++          ; read the <pattern> byte         |B0
        movwf   PORTB           ; display the pattern             |B0
        moviw   FSR1++          ; read the <duration> byte        |B0
        movwf   duration        ; set the duration                |B0
;
;  sequence timer for current step (sample push button switch)
;
seqtmr
        DelayCy(10*msecs-10)    ; 10-ms minus 10 cycle loop time  |B0
;
;   wreg  ____---____-----_____   inverted active lo sample
;  swold  _____---____-----____   switch state latch
;   wreg  ____-__-___-____-____   changes, press or release
;   wreg  ____-______-_________   filter out 'release' bits
;
        comf    PORTA,W         ; sample active lo switches       |B0
        andlw   1<<sw1          ; on the sw1 (RA0) pin only       |B0
        xorwf   swold,W         ; changes, press or release       |B0
        xorwf   swold,F         ; update switch state latch       |B0
        andwf   swold,W         ; filter out 'release' bits       |B0
        skpz                    ; new press? no, skip, else       |B0
        bra     nextfx          ; branch (do next effect)         |B0
        decfsz  duration,F      ; done? yes, skip, else           |B0
        bra     seqtmr          ; loop (another 10 msecs)         |B0
;
;  check for end of current sequence
;
        decfsz  steps,F         ; last step? yes, skip, else      |B0
        bra     newstep         ; do next step in sequence        |B0
        bra     newseq          ; restart current sequence        |B0

;******************************************************************
;  K8LH DelayCy() subsystem 16-bit uDelay subroutine              *
;******************************************************************
        nop                     ; entry for (delay-11)%5 == 4     |B0
        nop                     ; entry for (delay-11)%5 == 3     |B0
        nop                     ; entry for (delay-11)%5 == 2     |B0
        nop                     ; entry for (delay-11)%5 == 1     |B0
uDelay  addlw   -1              ; subtract 5 cycle loop time      |B0
        skpc                    ; borrow? no, skip, else          |B0
        decfsz  delayhi,F       ; done?  yes, skip, else          |B0
        goto    uDelay          ; do another loop                 |B0
        return                  ;                                 |B0

;******************************************************************
;  directory of fx table addresses                                *
;******************************************************************

address macro target
        retlw   target%256      ; table address lo byte
        retlw   target/256      ; table address hi byte
        endm

fxdir
        address(fx01)           ; fx01 array address
        address(fx02)           ; fx02 array address
        address(fx03)           ; fx03 array address
        address(fx04)           ; fx04 array address
        address(fx05)           ; fx05 array address
        address(fx06)           ; fx06 array address
        address(fx07)           ; fx07 array address
        address(fx08)           ; fx08 array address
        address(fx09)           ; fx09 array address
        address(fx10)           ; fx10 array address
        address(fx11)           ; fx11 array address
        address(fx12)           ; fx12 array address
        address(fx13)           ; fx13 array address
;
;  add additional fx array/table address entries at the end of
;  the 'fxdir' table (127 entries maximum).  the 'fxend' label
;  must immediately follow the last address entry.
;        
fxend
fxtot   equ (fxend-fxdir)/2     ; total number of fx arrays

;******************************************************************
;  fx (effects) arrays/tables                                     *
;******************************************************************

fx01
        dt      (fx02-fx01)/2   ; <steps>                         |B0
        dt      b'00000001',10  ; <pattern>, <duration>           |B0
        dt      b'00000011',10  ;                                 |B0
        dt      b'00000110',10  ;                                 |B0
        dt      b'00001100',10  ;                                 |B0
        dt      b'00011000',10  ;                                 |B0
        dt      b'00110000',10  ;                                 |B0
        dt      b'01100000',10  ;                                 |B0
        dt      b'11000000',10  ;                                 |B0
        dt      b'10000000',10  ;                                 |B0
        dt      b'00000000',20  ;                                 |B0
fx02
        dt      (fx03-fx02)/2   ; <steps>                         |B0
        dt      b'00000001',8   ; <pattern>, <duration>           |B0
        dt      b'00000010',8   ;                                 |B0
        dt      b'00000100',8   ;                                 |B0
        dt      b'00001000',8   ;                                 |B0
        dt      b'00010000',8   ;                                 |B0
        dt      b'00100000',8   ;                                 |B0
        dt      b'01000000',8   ;                                 |B0
        dt      b'10000000',8   ;                                 |B0
        dt      b'01000000',8   ;                                 |B0
        dt      b'00100000',8   ;                                 |B0
        dt      b'00010000',8   ;                                 |B0
        dt      b'00001000',8   ;                                 |B0
        dt      b'00000100',8   ;                                 |B0
        dt      b'00000010',8   ;                                 |B0
fx03
        dt      (fx04-fx03)/2   ; <steps>                         |B0
        dt      b'00001111',20  ; <pattern>, <duration>           |B0
        dt      b'11110000',20  ;                                 |B0
fx04
        dt      (fx05-fx04)/2   ; <steps>                         |B0
        dt      b'00000001',25  ; <pattern>, <duration>           |B0
        dt      b'00000010',25  ;                                 |B0
fx05
        dt      (fx06-fx05)/2   ; <steps>                         |B0
        dt      b'10000000',10  ; <pattern>, <duration>           |B0
        dt      b'11000000',10  ;                                 |B0
        dt      b'01100000',10  ;                                 |B0
        dt      b'00110000',10  ;                                 |B0
        dt      b'00011000',10  ;                                 |B0
        dt      b'00001100',10  ;                                 |B0
        dt      b'00000110',10  ;                                 |B0
        dt      b'00000011',10  ;                                 |B0
        dt      b'00000001',10  ;                                 |B0
        dt      b'00000000',20  ;                                 |B0
fx06
        dt      (fx07-fx06)/2   ; <steps>                         |B0
        dt      b'10000000',4   ; <pattern>, <duration>           |B0
        dt      b'01000000',4   ;                                 |B0
        dt      b'00100000',4   ;                                 |B0
        dt      b'00010000',4   ;                                 |B0
        dt      b'00001000',4   ;                                 |B0
        dt      b'00000100',4   ;                                 |B0
        dt      b'00000010',4   ;                                 |B0
        dt      b'00000001',8   ;                                 |B0
        dt      b'10000001',4   ;                                 |B0
        dt      b'01000001',4   ;                                 |B0
        dt      b'00100001',4   ;                                 |B0
        dt      b'00010001',4   ;                                 |B0
        dt      b'00001001',4   ;                                 |B0
        dt      b'00000101',4   ;                                 |B0
        dt      b'00000011',8   ;                                 |B0
        dt      b'10000011',4   ;                                 |B0
        dt      b'01000011',4   ;                                 |B0
        dt      b'00100011',4   ;                                 |B0
        dt      b'00010011',4   ;                                 |B0
        dt      b'00001011',4   ;                                 |B0
        dt      b'00000111',8   ;                                 |B0
        dt      b'10000111',4   ;                                 |B0
        dt      b'01000111',4   ;                                 |B0
        dt      b'00100111',4   ;                                 |B0
        dt      b'00010111',4   ;                                 |B0
        dt      b'00001111',8   ;                                 |B0
        dt      b'10001111',4   ;                                 |B0
        dt      b'01001111',4   ;                                 |B0
        dt      b'00101111',4   ;                                 |B0
        dt      b'00011111',8   ;                                 |B0
        dt      b'10011111',4   ;                                 |B0
        dt      b'01011111',4   ;                                 |B0
        dt      b'00111111',8   ;                                 |B0
        dt      b'10111111',4   ;                                 |B0
        dt      b'01111111',8   ;                                 |B0
        dt      b'11111111',50  ;                                 |B0
        dt      b'00000000',50  ;                                 |B0
fx07
        dt      (fx08-fx07)/2   ; <steps>                         |B0
fx08
        dt      (fx09-fx08)/2   ; <steps>                         |B0
fx09
        dt      (fx10-fx09)/2   ; <steps>                         |B0
fx10
        dt      (fx11-fx10)/2   ; <steps>                         |B0
fx11
        dt      (fx12-fx11)/2   ; <steps>                         |B0
fx12
        dt      (fx13-fx12)/2   ; <steps>                         |B0
fx13
        dt      (fxx-fx13)/2    ; <steps>                         |B0
;
;  this 'fxx' label must remain the last label in the file.
;
fxx

;******************************************************************
        end


